/** \file extras.hh
 * Extras for string composition library. */
/* Copyright (C) 2006-2007 Kevin Daughtridge <kevin@kdau.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#ifndef COMPOSE__EXTRAS_HH
#define COMPOSE__EXTRAS_HH

#include <ctime>
#include <sstream>
#include <string>
#include <glibmm/ustring.h>

/******************************************************************************/
namespace compose {
/******************************************************************************/

using std::string;
using Glib::ustring;

/** Decompose a string containing a single representation to its object type. */
template<typename T> inline T
decompose(const string &src, const T &fallback = T())
{
	std::istringstream is; is.str(src);
	T result(fallback); is >> result;
	return result;
}

/** Compose a single object into its string representation. */
template<typename T> inline string
compose1(const T &src, int precision = 0)
{
	std::ostringstream os;
	if (precision > 0) os.precision(precision);
	os << src; return os.str();
}

/** Compose a single object into its UTF-8 string representation. */
template<typename T> inline ustring
ucompose1(const T &src, int precision = 0)
{
	std::ostringstream os;
	if (precision > 0) os.precision(precision);
	os << src; return os.str();
}

/** Compose a string from a time value using libc's strftime safely.
 * \param fmt A strftime-style format string.
 * \param time The time to be represented, in seconds since the epoch.
 * \param local Whether to represent local (<tt>true</tt>) or UTC
 * (<tt>false</tt>) time.
 * \param fallback String to return if buffer space cannot be allocated. By
 * default, the empty string. */
string strftime(const string &fmt, time_t time, bool local,
	const string &fallback = string());

} /* namespace compose */

#endif /* COMPOSE__EXTRAS_HH */
