/** \file table.hh
 * Functions for access to the entire table. */
/*
 * This file is part of libgelemental, a periodic table library with detailed
 * information on elements.
 *
 * Copyright (C) 2006-2007 Kevin Daughtridge <kevin@kdau.com>
 * Copyright (C) 2003 Jonas Frantz <jonas.frantz@welho.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef LIBGELEMENTAL__TABLE_HH
#define LIBGELEMENTAL__TABLE_HH

#include <libgelemental/value.hh>
#include <libgelemental/value-types.hh>
#include <libgelemental/element.hh>

#include <stdexcept>

/******************************************************************************/
namespace gElemental {
/******************************************************************************/

/** The periodic table of elements. */
typedef std::vector<const Element*> Table;

/** Initializes libgelemental.
 * This function may be called more than once. This function, or one of the
 * get_table() or get_element() functions, which call it, should be called
 * before any other features in the library are used. */
void initialize();

/** Returns the periodic table of elements.
 * The table is a singleton. */
const Table& get_table();

/** Returns the element with a given atomic number.
 * \param number The atomic number of an element.
 * \return A reference to the corresponding element.
 * \throw std::out_of_range if the atomic number is invalid. */
const Element& get_element(int number) throw(std::out_of_range);

/** Returns the element with a given symbol or atomic number.
 * \param which The standard symbol of an element, or the string representation
 * of the atomic number of an element.
 * \return A reference to the corresponding element.
 * \throw std::invalid_argument if the symbol or number is invalid. */
const Element& get_element(const ustring& which) throw(std::invalid_argument);

} /* namespace gElemental */

#endif /* LIBGELEMENTAL__TABLE_HH */
